/// <summary>
/// Frame to show a modern Color-selection box
/// <img src="FrameColorpicker.jpg">---<img src="FrameColorpicker2.jpg">
/// Documentation generated by <a href="https://www.hastasoft.de/PPME.htm">Pascal Project Manager & Editor</a>
/// </summary>
/// <remarks>
/// How to use: 
/// 1 Add the unit to your project
/// 2 From the Component-selector select frames and draw a small rectangle on
///   the form, were the frame sould be insert
/// 3 In the opened dialog select FrameColorpicker
///
/// Downloadlink for source: <a href="https://www.devpage.de/FrameColorPicker.zip">FrameColorPicker.zip</a>
/// </remarks>
unit FrameColorPicker;

interface

uses
  System.SysUtils,
  System.Types,
  System.UITypes,
  System.Classes,
  System.Variants,

  FMX.Types,
  FMX.Graphics,
  FMX.Controls,
  FMX.Forms,
  FMX.Dialogs,
  FMX.StdCtrls,
  FMX.Objects,
  FMX.Colors,
  FMX.Layouts,
  FMX.Controls.Presentation;

type
  /// <summary>
  /// Here FRAME for the HTML-Editor window (but can be used also in any other projects)
  /// </summary>
  TF_FrameColorPicker = class(TFrame)
    ComboColorBox: TComboColorBox;     // A standard FMX ComboColorbox (only button is used)
    LayoutColorPickerButton: TLayout;  // Only container
    LayoutColorPickerColors: TLayout;
    FMXColorBox: TColorBox; // A standard FMX Colorbox (only used for the alpha-color part
    /// <summary>
    /// The image with the colorbox rectangles
    /// </summary>
    ColorBoxImage: TImage;
    LayoutColorButton: TLayout;
    /// <summary>
    /// Contains a standard FMX-Colorbox and the image
    /// </summary>
    RectangleColorBoxContainer: TRectangle;
    /// <summary>
    /// Click on color-button to show the colorbox
    /// </summary>
    procedure LayoutColorButtonClick(Sender: TObject);
    /// <summary>
    /// Returns the selected color and sets the comboboxColor (here only for the alpha-case)
    /// </summary>
    procedure FMXColorBoxClick(Sender: TObject);
    /// <summary>
    /// Returns the clicked color from the colorbox-image
    /// </summary>
    procedure ColorBoxImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
    /// <summary>
    /// Checks if ESC is pressed to close the color-box
    /// </summary>
    procedure FrameKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
    /// <summary>
    /// Hide the Colorpicker (box)
    /// </summary>
    /// <example>
    /// <code>
    /// // If Picker is not closed by select or ESC,
    /// // check, when other controls been entered, if the picker is visible
    /// // and close it then
    /// // in a Form you could use also the event OnFocusChanged
    ///
    /// procedure TF_MainForm.CheckPickers;
    /// begin
    ///   FrTextColorPicker.HidePicker;
    ///   FrBGColorPicker.HidePicker;
    /// end;
    ///
    /// // if the frame is used in a form, you can use also
    /// // "Onfocus-Changed"-Event to make the check
    ///
    /// procedure TF_Main.FormFocusChanged(Sender: TObject);
    /// var
    ///   Cont: TControl;
    ///  begin
    ///    Cont := TControl (Focused.GetObject);
    ///
    ///    if Cont <> NIL then begin
    ///      if Cont <> F_FrameTextColorPicker.ComboColorBox then begin
    ///        F_FrameTextColorPicker.HidePicker;
    ///      end;
    ///
    ///      if Cont <> F_FrameBGColorPicker.ComboColorBox then begin
    ///        F_FrameBGColorPicker.HidePicker;
    ///      end;
    ///    end;
    ///  end;
    /// </example>
    /// </code>
    procedure HidePicker;
  private
  public
    /// <summary>
    /// If this is true, then the part for Alpha-color is not shown (see image 1), so set it on creation event
    /// </summary>
    HideAlpha: Boolean;
  end;

implementation

{$R *.fmx}

procedure TF_FrameColorPicker.FMXColorBoxClick(Sender: TObject);
begin
  LayoutColorPickerColors.Visible := false;
  ComboColorBox.color := FMXColorbox.Color;
end;

procedure TF_FrameColorPicker.FrameKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
begin
  if Key = VKEscape then begin
    LayoutColorPickerColors.Visible := false;
  end;
end;

procedure TF_FrameColorPicker.HidePicker;
begin
  if LayoutColorPickerColors.Visible then LayoutColorPickerColors.Visible := false;
end;

procedure TF_FrameColorPicker.ColorBoxImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
var
  vBitMapData : TBitmapData;  vPixelColor : TAlphaColor;
begin
  if ColorBoxImage.Bitmap.Map(TMapAccess.Read, vBitMapData) then
  begin
    vPixelColor := vBitmapData.GetPixel(round (x), round (y));
    ComboColorBox.color := vPixelColor;
    ColorBoxImage.Bitmap.Unmap(vBitMapData);
  end;

  LayoutColorPickerColors.Visible := false;
end;

procedure TF_FrameColorPicker.LayoutColorButtonClick(Sender: TObject);
var
  add: Single;
begin
  add := 0;

  FMXColorBox.Visible := not HideAlpha;

  if HideAlpha then begin
    add := FMXColorbox.Height;
  end;

  LayoutColorPickerColors.Visible := Not LayoutColorPickerColors.Visible;
  LayoutColorPickerColors.height := ColorBoxImage.Height + FMXColorbox.Height-add;
  RectangleColorBoxContainer.position.X := 0; RectangleColorBoxContainer.position.y := 0;

  LayoutColorPickerButton.Parent.BringToFront;
  LayoutColorPickerColors.BringToFront;
end;

end.
